/*
 * irdecode.h
 *
 *  Created on: 5 Jul 2012
 *      Author: mmaher
 *
 *  IR Decode
 *  =========
 *  Apple Remote uses NEC IR format which is 32bit PWM with additional start/stop pulses:
 *  <START PULSE> <8bit Addr> <8bit addr inverted> <8bit data> <8bit data inverted> <STOP PULSE>
 *
 *  START is 9ms pulse followed by 4.5ms space
 *  DATA bits are 0 = 562.5s pulse followed by 562.5s space (1.125ms total)
 *                1 = 562.5s pulse followed by 1.6875ms space (2.25ms total)
 *  STOP is 562.5s pulse
 *
 * In addition to this, a different repeat pulse is also provided:
 *  <REPEAT> <STOP>
 *   REPEAT is 9ms high followed by 2.25ms space
 *   STOP is the same - 562.5us high
 *
 *  Because of the inverted signal, this means total normal transmission time is always the same:
 *  (8x 1.125ms + 8x 2.25ms) x2 + 13.5ms (ignore stop time) = 67.5ms
 *
 *  we're feeding the timer from SMCLK at 1MHZ, divided by 8 = 125kHz = 8uS
 *   - which makes total transmission = 67.5ms / 8us = 8437.5counts
 *   - 0 cycle is 1.125ms = 140.6 counts
 *   - 1 cycle is 2.25ms = 281.25 counts
 *   - start cycle 13.5ms = 1687.5 counts
 *   - repeat cycle 11.25ms = 1406.25 counts
 *
 *  Capture works as follows:
 *   - ISR is setup to trigger on each falling edge.
 *   - As soon as the ISR is raised, the counter timer is read & reset
 *   - counts are then compared from the table above to determine what's what
 *   NOTE: we are fuzzy-matching these counts since it's essentially a 38kHz signal being sampled
 *   from a 32.768kHz clock source - it's not a precision art!
 *
 * Apple remote codes aren't 100% NEC code compliant - inverse words aren't inverse.
 *
 * My test apple remote (white A1156) set to factory default returns:
 *  Play/Pause  = EE 87 05 5E
 *  +		 	= EE 87 0A 5E //11101110  10000111 00001010 01011110
 *  -		 	= EE 87 0C 5E
 *  Left	 	= EE 87 09 5E
 *  Right	 	= EE 87 06 5E
 *  Menu	 	= EE 87 03 5E
 */

#ifndef IRDECODE_H_
#define IRDECODE_H_

#include "globals.h"

#define IR_BIT1_APPLE 0xEE
#define IR_BIT2_APPLE 0x87
#define IR_BIT4_APPLE 0x61//not 5E anymore, updated remote0x5E
#define IR_BIT3_VOLUP 0x0A
#define IR_BIT3_VOLDN 0x0C
#define IR_BIT3_PLAY  0x05
#define IR_BIT3_MENU  0x03

#define ZERO_TIME 136 //141 is the calculated value
#define ONE_TIME 272 //281 is the calculated value
#define START_TIME 1672 //1687 is the calculated value, but it never hits the spot
#define REPEAT_TIME 1388 //1406 is the calculated value
#define TOLERANCE 8

#define ZERO_LOWER (ZERO_TIME - TOLERANCE)
#define ZERO_UPPER (ZERO_TIME + TOLERANCE)
#define ONE_LOWER (ONE_TIME - TOLERANCE)
#define ONE_UPPER (ONE_TIME + TOLERANCE)
#define REPEAT_LOWER (REPEAT_TIME - TOLERANCE)
#define REPEAT_UPPER (REPEAT_TIME + TOLERANCE)
#define START_LOWER (START_TIME - TOLERANCE)
#define START_UPPER (START_TIME + TOLERANCE)

/* GLOBALS */
typedef enum
{
   eIDLING,
   eDATACAPTURED,
   eDATAREPEAT,
   eERROR
} MAIN_STATE;

typedef enum
{
   eIDLE,
   eSTART,
   eFIRSTBIT,
   eSECONDBIT,
   eTHIRDBIT,
   eFOURTHBIT,
   eCLOSE
} TIMER_STATE;
TIMER_STATE tState;

typedef enum
{
  ePulseNOISE,
  ePulseZERO,
  ePulseONE,
  ePulseSTART,
  ePulseSTOP,
  ePulseREPEAT,
  ePulseERROR
} PULSE_VAL;
PULSE_VAL LatestPulse;
//#define TESTDATA

unsigned char irData[6];

#ifdef TESTDATA
unsigned char testData[38];  //should be 34 to get full coverage
unsigned char testDataIndex;
//unsigned char errorCount;
#endif


MAIN_STATE IRStateMachine(PULSE_VAL val)
{
	MAIN_STATE mState = eIDLING;
	static unsigned char bitcount = 0;

	switch (tState)
	{
	case eIDLE:	//this means we've got a transmission incoming, change state
		tState++;//eSTARTSPACE
		break;
	case eSTART:
		switch (val)
		{
		case ePulseSTART:
			P1OUT &= ~LED;// Turn out the LED
			bitcount = 0;
			tState++;
			break;
		case ePulseREPEAT:	//flag for valid data and quit state machine
			mState = eDATAREPEAT;
			goto end;
		default:
			mState = eERROR;
			goto end;
		}

		break;
	case eFIRSTBIT:
	case eSECONDBIT:
	case eTHIRDBIT:
	case eFOURTHBIT:
		switch (val)
		{
		case ePulseONE:		irData[tState] |= (1 << bitcount); 		break;
		case ePulseZERO:	irData[tState] &= ~(1 << bitcount);		break;
		default:
			mState = eERROR; //flag erroneous reception to shut down any additional repeats
			goto end;

		}
		bitcount++;
		if (bitcount == 8)
		{
			bitcount = 0;
			if (tState == eFOURTHBIT)
			{//job done, won't see stop as no more falling edges to trigger on
				mState = eDATACAPTURED; //flag valid data and exit sm
				goto end;
			}
			else
				tState++;
		}
		break;
	default:
		goto end;
	}

	return mState;
	end:
		bitcount = 0;
		P1OUT |= LED;// Light the LED
		tState = eIDLE;
		return mState;

}

//----------------------------------------------------------------------------
void TimerA2_ISR(void) // Timer_A CCR0 Interrupt Handler
{

	unsigned count = TACCR0;
	//TAR = 0; //reset count for each incoming interrupt
	TACTL |= TACLR; //reset counter

#ifdef TESTDATA
	testData[testDataIndex++] = count / 8; //div 8 to fit into byte size
#endif


	//decode timer counter into one of 5 possibilities
	if ((count > ZERO_LOWER) && (count < ZERO_UPPER))
		LatestPulse = ePulseZERO;
	else if ((count > ONE_LOWER) && (count < ONE_UPPER))
		LatestPulse = ePulseONE;
	else if ((count > START_LOWER) && (count < START_UPPER))
		LatestPulse = ePulseSTART;
	else if ((count > REPEAT_LOWER) && (count < REPEAT_UPPER))
		LatestPulse = ePulseREPEAT;
	else
		LatestPulse = ePulseERROR;

}

void TimerOverflow_ISR(void)
{
	//emergency reset to stop random pulses getting the system into a silly state
	P1OUT |= LED;// Light the LED
	tState = eIDLE;
}


#endif /* IRDECODE_H_ */
