/*
 * wm8718.h
 *
 *  Created on: 6 Jul 2012
 *      Author: mmaher
 *
 *  DAC Control:
 *
 *  Left and right attenuation is controlled via the LATTN/RATTN registers.
 *  Bits 7..0 control the attenuation in 0.5db steps, with the default being 1111111=0dB
 *  Hence first we need to set an initial attenuation to reasonable level, not full blast.
 *
 *  Bit 8 of the attn registers is the "update now or later" flag. For the purposes of ou setup, just update now - which is 1 (0x80)
 *
 *  The mode control field consists of
 *     Bit Function
 *     0   Mute (set to 0)
 *     1   Deemph (set to 0) //mm change 24/08 was 1, set to 0 - richard hawley flag!
 *     2   Pwrdwn (set to 0)
 *     7   Zero flag control (set to 1)
 *     8   zero cross detect (set to 1 disabled)
 *     9-16 fixed address 2
 *
 *     default: 0000 0101 1000 0000 (0x0580)
 *
 *  Finally format control:
 *     Bit Function
 *     0:1 input word format:
 *              00 16bit mode <<
 *              01 20bit mode
 *              10 24bit mode
 *              11 32bit mode
 *     2:3 audio format:
 *              00 right justified
 *              01 left justified
 *              10 i2s mode <<
 *              11 dsp mode
 *     4   polarity select for DSP mode (0=normal)
 *     5   attn control (Set to 1 to mirror right dac with left)
 *     6   BCKIN polarity (0 for normal)
 *     7   o/p phase reversal (0 for normal)
 *
 *     default: <WAS> 0000 0110 0010 1000 (0x0628)
 *     default: <NOW> 0000 0110 0010 1001 (0x0629)
 */

#ifndef WM8718_H_
#define WM8718_H_
#include "globals.h"

#define DAC_REG_LATTN 0x00
#define DAC_REG_RATTN 0x01
#define DAC_REG_MODE  0x02
#define DAC_REG_FORMAT 0x03

#define DAC_DEFAULT_VOL 164 //255 is full blast
#define DAC_LATTN_UPDATE 0x0100
#define DAC_RATTN_UPDATE 0x0300
#define DAC_DEFAULT_LATTN (DAC_LATTN_UPDATE | DAC_DEFAULT_VOL) //0000 0001 xxxx xxxx
#define DAC_DEFAULT_RATTN (DAC_RATTN_UPDATE | DAC_DEFAULT_VOL) //0000 0011 xxxx xxxx
#define DAC_DEFAULT_MODE 0x0580 //mm change from 582
#define DAC_DEFAULT_FORMAT 0x0629 ///was 628

#define WAIT_CLKS 29

unsigned char attenuation;
unsigned char togglemute;
unsigned char togglehawleyflag;
unsigned DACMode;

//special SPI write function for the wm8718 which is a bit odd
void SPIwriteDAC(unsigned data)
{
	//load data
	USISR = data;

	USICNT |= 16; //increment count to start TX

	//latch line needs to be dropped on the last bit, so wait around..
	//SPI clocks at 2mhz, same as processor. Hence 30 processor ticks *should* work, minus bumph
	__delay_cycles(WAIT_CLKS);

	P1OUT &= ~DAC; //drop latch
	P1OUT |= DAC; //raise latch

	//final test to make sure serial is clear before exiting
	while (!(USIIFG & USICTL1));
}

void initDAC()
{
    //raise SPI latch line
    P1OUT |= DAC;
    __delay_cycles(65535);

    attenuation = DAC_DEFAULT_VOL;
    togglemute = 0;
    togglehawleyflag = 0;
    DACMode = DAC_DEFAULT_MODE;

    SPIwriteDAC(DAC_DEFAULT_MODE);
    SPIwriteDAC(DAC_DEFAULT_FORMAT);
    SPIwriteDAC(DAC_DEFAULT_LATTN);
    SPIwriteDAC(DAC_DEFAULT_RATTN);
}

void volumeUp()
{
	if (attenuation < 254)
	{
		attenuation+=2;

		//only need to update left attenuator as mirroring set
		SPIwriteDAC(DAC_LATTN_UPDATE | attenuation);
//		SPIwriteDAC(DAC_RATTN_UPDATE | attenuation);
	}
}

void volumeDown()
{ //volume down = attenuation up
	if (attenuation > 1)
	{
		attenuation-=2;

		//only need to update left attenuator as mirroring set
		SPIwriteDAC(DAC_LATTN_UPDATE | attenuation);
		//SPIwriteDAC(DAC_RATTN_UPDATE | attenuation);
	}
}

void mute()
{
	if (togglemute == 0) //off
	{
		DACMode |= 0x0001;
		togglemute = 1;
	}
	else
	{
		DACMode &= 0xFFFE;
		togglemute = 0;
	}
	SPIwriteDAC(DACMode);
}

void richardHawleySetting()
{
	//the richard hawley setting is actually the digital de-emphasis setting
	// - so called because you need this set when listening to some richard hawley tracks otherwise it gets painful
	if (togglehawleyflag == 0) //off
	{
		DACMode |= 0x0002;
		togglehawleyflag = 1;
	}
	else
	{
		DACMode &= 0xFFFD;
		togglehawleyflag = 0;
	}
	SPIwriteDAC(DACMode);
}


#endif /* WM8718_H_ */
